; Address Solar Light - July 14, 2010 
; Updated --- December 22, 2012 (single flashes for number readout instead of tripple flashes)

;Internal RC oscillator runs at 8 MHz / 8 = 1 MHz (default fuse settings)


;Wayne Simister
;Automated Sound
;584 Cortez Street
;Salt Lake City, UT 84103
;wsimister@hotmail.com


;ALL entries are a dit, dah, or hold down push of button.  Dit=1, Dah=10, Hold down= 50


;NOTE: Some entries use a +1 entry method to allow an entry to handle entry of 1 = 0)


;Following fuses are set on ATTiny2313 chip:
;  Divide by 8 to reduce clock frequency to 1,000,000 Hz


.include "tn2313def.inc"

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set Port A, bits 0-2 for output - no crystal, so port A active
out		DDRA,r16	;Execute data direction with Port A 0-2 as output

clr		r16
out		PortA,r16	;Turn off ports

ldi		r16,126		;Set bits PD1-6 to ones for out, bit PD0 for input
out		DDRD,r16	;Execute data direction with PD0 as input rest as output

clr		r16
out		PortD,r16	;All output ports low - LEDs off
					;No input resistor for input port D,0

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16

out		PORTB,R16	;execute to set out ports low

rcall	Delay4seconds	;to allow electronics to settle


;-------------------------------------

MainLoop:		;flashes out home address from EEPROM data

rcall	NumberFlashOut
rcall	EEDelay2sec
sbis	PIND,0
rcall	ExitForCommand


ML2:

rcall	MorseFlashOut
rcall	EEDelay4sec
sbis	PIND,0
rcall	ExitForCommand

rjmp	MainLoop


;----------------------------------------

ExitForCommand:

push	r16
push	r17
push	r20
push	r21
push	r22
push	r23
push	r30

rcall	ButtonStall
rcall	Decode

pop		r30
pop		r23
pop		r22
pop		r21
pop		r20
pop		r17
pop		r16

ret


;----------------------------------------

Decode:				;decodes complete count in r16

cpi		r16,11		;Is it good for entering numbers?
brne	DE12

rcall	NumberEntry
rjmp	DEEnd


DE12:

cpi		r16,12		;Is it good for entering Morse entries?
brne	DE15

rcall	MorseEntry
rjmp	DEEnd


DE15:

cpi		r16,15
brne	DEEnd

rcall	Enter584Cortez
rjmp	DEEnd


DEend:

ldi		r16,7
rcall	FastWhiteLEDflash	;flash LEDs 7 times

ret


;--------------------------------------


WriteEEPROM:		;write new code into EEPROM
					;r16 has data
					;r30 has memory location 
WEE:

sbic	EECR,EEPE	;wait until EEPROM ready for new entry
rjmp	WEE

out		EEAR,r30	;set up address in address register
out		EEDR,r16	;put contents of r16 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

ret


;--------------------------------------------------

FetchEEPROM:		;grabs data in EEPROM, r30 address location
					;r16 will contain data byte

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r30	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r16,EEDR	;read data from data register

ret


;-------------------------------------------

ButtonStall:		;Use this entrance when you wish to wait for a button push.
					;  It goes to Button when switch goes down.
					;NOTE:  This coding same as used for OBH2a 6/27/2010 with modifications for i/o ports

clr		r16
out		PortB,r16	;OFF all LEDs

sbis	PIND,0		;skip if switch is up
rjmp	Button

rjmp	ButtonStall


;---

ButtonStallFlash:	;Rapidly flashes blue LED when waiting for command.
					; Stalls and waits for command entry.


BSF1:

sbi		PortB,0		;ON blue LED
rcall	EEDelay40th
sbis	PIND,0		;skip if PIND,0 - swith is up
rjmp	Button

cbi		PortB,0		;OFF blue LED
rcall	EEDelay10th
sbis	PIND,0		;skip if PIND,0 - switch is up
rjmp	Button

rjmp	BSF1


;---

Button:		;Takes in dit, dah, holddown, and coded input 

clr		r16
out		PortB,r16	;OFF all LEDs

push	r0		;number of code digits
push	r1		;low code byte
push	r2		;next high code byte
push	r3		;next high code byte
push	r4		;highest code byte
push	r18		;dit count
push	r20		;holddown count
push	r21		;Max code count via shift right of bit
push	r23		;universal counter
push	r28		;dah count
push	r30		;has total count of entry in Z register, r30,31
push	r31

;r30,r31 (Z register pair contain total value of entry)

clr		r18		;Dit Count
clr		r20		;HoldDown count
clr		r21		;Max code via shift right of bit
clr		r23		;universal counter
clr		r28		;dah count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)


				;r30,31 has total count

B1:

inc		r23			;universal counter
rcall	Delay40th
cpi		r23,60		;delay before first holddown count - 40=one second 
brsh	B3			;branch if - it is a holddown count

sbis	PIND,0		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

rcall	MaxCountInR21

inc		r28			;increment dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0		;on white LED
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th
cbi		PortB,0		;off LED
clr		r23
inc		r0			;increment number of code digits


B2d:				;no touch/switch up detection

sbis	PIND,0		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,85		;has switch been up 85 counts? 40=one second (2 + seconds)
brsh	B4Decode1
rjmp	B2d


BPEnd1:

rjmp	BPEnd


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

rcall	MaxCountInR21

inc		r18			;increment dit count

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0

inc		r0			;increment number of code digits (r0)
clr		r23			;clear r23
rjmp	B2d


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortB,0
rcall	Delay40th
rcall	Delay40th
cbi		PortB,0		;flash RED LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,0		;skip if switch is down
rjmp	B4a			;jump out and look for next entry

cpi		r23,30		;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d			;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31			;ready registers for input

mov		r16,r28		;move dah count in r28 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10		;add value of 10 in r30 to r314
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r18		;add value in r18 (dit count)
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31			;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r20		;put hold down count into r16


Bholdown1b:

tst		r16			;test r20(r16), hold down count
breq	BPEnd
adiw	r30,50		;add fifty count to total
dec		r16
rjmp	Bholdown1b

;Complete combined value now in Z register pair


BPEnd:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$7C,r18		;number of dits
sts		$7D,r28		;number of dahs
sts		$7E,r20		;number of hold down counts
sts		$7F,r30		;LSD of total count
sts		$80,r31		;MSD of total count
sts		$81,r1		;Code (8 digits only)
sts		$82,r21		;Max Location.  Combined dit dah shifts of dit dah count
						; (Example: 5 shifts = 00010000)-Max code count 
sts		$83,r0		;Number of code digits (dit and dahs)

pop		r31
pop		r30
pop		r28
pop		r23
pop		r21
pop		r20
pop		r18
pop		r4
pop		r3
pop		r2
pop		r1
pop		r0

lds		r16,$7F		;put low byte (total count) into r16

ret


;-------------------------------

MaxCountInR21:

tst		r21			;check to see if this is first entry for string of dit/dah entries
breq	B2p1		;This section rotates a bit left for each dit/dah entry (max bit position entry)

rjmp	B2p2


B2p1:				;This is first iteration of new entry

ldi		r21,1		;Set first right bit on
rjmp	B2p3


B2p2:				;all other entries will rotate bit one place to left

lsl		r21			;logical right shift of bit in r21 (max number of bit entries)


B2p3:	

ret


;------------------

;EEDelay Routines --- EXIT EARLY -- Quickly exits upon Button Push

;------------------

EEDelay800th:			;BASE delay for other early exit delays

push	r16
ldi		r16,247


EED800a:

sbis	PIND,0			;jump out of delay if switch goes down (low)
rjmp	EED800end

dec		r16
brne	EED800a


EED800end:

pop	r16

ret


;---------------

EEDelay40th:

push	r16
ldi		r16,20


EEDelay40a:

sbis	PIND,0			;jump out of delay if switch goes down (low)
rjmp	EEDelay40end

rcall 	Delay800th
dec		r16
brne	EEDelay40a


EEDelay40end:

pop		r16

ret


;--------------

EEDelay10th:			;DIT

push	r16
ldi		r16,80


EEDelay10a:

sbis	PIND,0			;jumpe out if switch down
rjmp	EEDelay10end

rcall	EEDelay800th
dec		r16
brne	EEDelay10a


EEDelay10end:

pop		r16
ret


;---------------

EEDelay3rd:				;DAH - 1/3 second)

push	r16
ldi		r16,240


EEDelay3rdA:

sbis	PIND,0			;jumpe out if switch down
rjmp	EEDelay3rdEnd

rcall	EEDelay800th
dec		r16
brne	EEDelay3rdA


EEDelay3rdEnd:

pop		r16
ret


;---------------

EEDelay500ms:

push	r16
ldi		r16,5


EEDelay500a:

sbis	PIND,0			;skip if switch down
rjmp	EEDelay500end

rcall	EEDelay10th
dec		r16
brne	EEDelay500a


EEDelay500end:

pop		r16

ret


;---------------

EEDelay300ms:


push	r16
ldi		r16,3


EEDelay300a:

sbis	PIND,0			;skip if switch down
rjmp	EEDelay300end

rcall	EEDelay10th
dec		r16
brne	EEDelay300a


EEDelay300end:

pop		r16

ret


;---------------

EEDelay1sec:

push	r16
ldi		r16,10


EEDelay1sec1:

sbis	PIND,0			;jump out if switch down
rjmp	EEDelay1end

rcall	EEDelay10th
dec		r16
brne	EEDelay1sec1


EEDelay1end:

pop		r16

ret


;---------------------------------------

EEDelay2sec:

push	r16
ldi		r16,20


EEDelay2sec1:

sbis	PIND,0			;jump out if switch is down
rjmp	EEDelay2secEnd

rcall	EEDelay10th
dec		r16
brne	EEDelay2sec1


EEDelay2secEnd:

pop		r16

ret


;------------------------------------------

EEDelay4sec:

push	r16
ldi		r16,40


EEDelay4sec1:

sbis	PIND,0			;jump out if switchis down
rjmp	EEDelay4secEnd

rcall	EEDelay10th
dec		r16
brne	EEDelay4sec1


EEDelay4secEnd:

pop		r16

ret


;---------------------------------------

; DELAY ROUTINES

;---------------------------------------

Delay800th:			;BASE delay for other delays

push 	r16
ldi		r16,247		;1246 total clock cycle delay (1250 was goal)


D800a:

dec		r16
nop
nop
brne	D800a

pop		r16

ret


;-------------------------------------

Delay40th:

push	r16
ldi		r16,20


D40a:

rcall 	Delay800th
dec		r16
brne	D40a

pop		r16

ret


;-----------------------------------------

Delay10th:			;1/10th second delay

push	r16
ldi		r16,80

rjmp	D40a


;-----------------------------------------

Delay4th:			;1/4 second delay

push	r16
ldi		r16,200

rjmp	D40a


;------------------------------------------

Delay4seconds:

push	r16
ldi		r16,40		;for 4 second delay


D4a:

rcall	Delay10th
dec		r16
brne	D4a

pop		r16

ret


;------------------------------------

FastWhiteLEDflash:		;r16 has number of flashes

sbi		PortB,0			;on white LED
rcall	Delay40th
cbi		PortB,0			;off white LED

rcall	Delay40th

dec		r16
brne	FastWhiteLEDflash

ret


;---------------------------------------

WhiteLEDflash:			;r16 contains number of flashes


sbi		PortB,0			;on White LED
rcall	Delay40th
cbi		PortB,0			;off LED

rcall	Delay4th		;1/4 second delay
rcall	Delay4th		;totsl 1/2 second delay


dec		r16
brne	WhiteLEDflash

ret


;-------------------------------------

FlashWhiteLED:		;r16 contains number of flashes

FWa:

sbi		PortB,0		;ON white led
rcall	Delay40th
cbi		PortB,0		;OFF white led

rcall	Delay4th
rcall	Delay4th
dec		r16
brne	FWa

ret


;-------------------------------------------

FlashOutR16:			;Data placed in r16 is flashed out with double flash = 1 (low to high)

push	r16				;save original data
push	r17				;scratch
ldi		r17,8

rcall	Delay4seconds


FOD1a:

sbrc	r16,0			;skip if zero bit in r16 is clear
rjmp	FOD1

sbi		PortB,0			;ON white LED
rcall	Delay40th
cbi		PortB,0			;OFF LED

rcall	Delay4th
rcall	Delay4th
rcall	Delay4th

rjmp	FOD2


FOD1:

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0

rcall	Delay40th
rcall	Delay40th
rcall	Delay40th

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0

rcall	Delay4th
rcall	Delay4th
rcall	Delay4th


FOD2:

dec		r17
breq	FODend

lsr		r16				;shift r16 right on digit to read next highest bit (low to high)
rjmp	FOD1a


FODend:

pop		r17
pop		r16
rcall	Delay4seconds

ret


;----------------------------------

LongFlash: 

sbi		PortB,0			;ON white LED
rcall	Delay4th
rcall	Delay4th
rcall	Delay4th
rcall	Delay4th
cbi		PortB,0			;OFF white LED

ret


;----------------------------------

NumberEntry:

; Ten locations in EEPROM, $00 through $0A.  $00 has number of number entries
; (Rest of EEPROM used for Morse Code entries = 116)
; r20 = number of entries counter
; r30 = address counter for EEPROM
; EXIT code when finished entering numbers = one hold down count


ldi		r30,$01			;start of EEPROM memory store for numbers
ldi		r20,1


NumEnt1:

rcall	ButtonStallFlash
dec		r16				;one less since 1 = 0, 2 = 1, etc. (Ex: 584 would be entry of 695)

lds		r17,$7E			;get number of hold down counts
cpi		r17,0

brne	NumEnt2

cpi		r20,11			;exit if number of entries are greater than 10
brsh	NumEnt2

rcall	WriteEEPROM		;put data into EEPROM
inc		r30
inc		r20

rjmp	NumEnt1


NumEnt2:

dec		r20
ldi		r30,$00			;for put in number of entries data location EEPROM

mov		r16,r20
rcall	WriteEEPROM

ret


;--------------------------------------

MorseEntry:

; r17 = scratch
; r20 = counter
; r21 = bit position
; r30 = EEPROM address

ldi		r30,$0C			;start location for all data, $0C start of codes,  $0B is number of entries
ldi		r20,1			;counter


Morse1:

rcall	ButtonStallFlash

lds		r17,$7E			;get hold down count
cpi		r17,0			
brne	Morse2			;EXIT if any count in Hold Down counter

cpi		r20,117
brsh	Morse2			;exit because number of codes has exceeded limnit

lds		r16,$83			;get entered number of code digits
cpi		r16,8			;Set entry to ZERO if entry exceeds limit
brsh	Morse4


;----------				;Install DAH in +1 position for code


ldi		r21,1			;Set extreme right bit in r21 - position
lds		r16,$83			;Get number of digits entered


Morse2a:

lsl		r21				;shift bit left
dec		r16
brne	Morse2a

lds		r16,$81			;get entered code
or		r16,r21			;install extra dah into r16 code


Morse3:

rcall	WriteEEPROM		;put data into EEPROM (r16 has new value)

inc		r30				;increment EEPROM location by one
inc		r20				;incrfement number of entries

rjmp	Morse1			;jump back for another round


;--------

Morse2:

dec		r20
mov		r16,r20

ldi		r30,$0B			;ready put total number of entries into $0B into EEPROM 

rcall	WriteEEPROM		;write to EEPROM - number of entries

ret


;-------

Morse4:

ldi		r16,1			;zero entry - no data to be entered
						;set dah at zero position

rjmp	Morse3


;---------------------------------------------------------

MorseFlashOut:			

;r20 = number of codes (characters in message)
;r21 = code
;r22 = Max position
;r23 = Max scratch

ldi		r30,$0B			;Start of Morse code section

rcall	FetchEEPROM		;r16 will have number of codes data
mov		r20,r16			;put fetched code in r20 (number of letters/spaces)


MF1:

tst		r20
brne	MF1a

sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

ret


MF1a:

inc		r30				;to next EEPROM location
rcall	FetchEEPROM
mov		r21,r16			;put Fetched value in r21 (code)

cpi		r21,128
brne	MF2

rcall 	EEDelay1sec		;Pause between Letters
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

dec		r20
rjmp	MF1				;go for another letter/pause


MF2:

ldi		r22,128			;set max position to far left in r22


MF4:

mov		r23,r22			;copy into r23
and		r23,r21			;check bit in code for max position
brne	MF4a			;max position +1 found

lsr		r22
rjmp	MF4


MF4a:					;code location +1 now found

lsr		r22
mov		r23,r22			;make copy in r23
and		r23,r21
brne	MF4b			;it is DAH


MF4c:					;it is DIT

sbi		PortB,0			;ON blue LED
rcall 	EEDelay10th
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

cbi		PortB,0			;OFF blue LED

rjmp	MF4d


MF4b:					;it is DAH

sbi		PortB,0			;ON blue LED
rcall 	EEDelay500ms
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

cbi		PortB,0			;OFF blue LED


MF4d:

rcall 	EEDelay500ms	;Pause between dit/dahs
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

cpi		r22,1			;if r22=1, this is end of letter/pause
brne	MF4a

dec		r20				;dec r20, number of codes count

rcall 	EEDelay1sec		;Pause between each letter
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

rjmp	MF1


;------------------------------------

NumberFlashOut:			;Reads out number entries (up to 10)

;r20 = Number of numbers in memory section
;r21 = data -- cout of DITS to be flashed out - numbers always flashed out in DITS

ldi		r30,$00			;start of number EEPROM memory section

rcall	FetchEEPROM
mov		r20,r16			;put fetched number into r20

cpi		r20,0
brne	NumFlash1

ret						;Return - no flashout necessary


NumFlash1:

inc		r30


NumFlash2:

rcall	FetchEEPROM
mov		r21,r16			;get fetched data value and put in r21, data 

cpi		r21,0
breq	NumFlash2a		;Zero flashout (long steady flash)


NumFlash2b:				;Singlele flash for each count

sbi		PortB,0			;ON blue LED - Send out flash 
rcall	EEDelay10th
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

cbi		PortB,0			;OFF blue LED
rcall	EEDelay40th
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

rcall	EEDelay3rd		;pause
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

dec		r21				;decrement number in data count
brne	NumFlash2b


NumFlash3:

rcall	EEDelay1sec
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

dec		r20				;number of numbers to be read out
brne	NumFlash1

rcall	EEDelay2sec
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

ret


NumFlash2a:				;Flashes out one long flash to designate a ZERO

sbi		PortB,0			;ON blue LED - to indicate a zero entry
rcall	EEDelay500ms
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

cbi		PortB,0			;OFF blue LED
rcall	EEDelay3rd		;call delay of 1/3 second
sbis	PIND,0			;skip if switch is up
rcall	ExitForCommand

rjmp	NumFlash3


;--------------------------------------------------

Enter584Cortez:

ldi		r30,$0			;start of EEPROM memory section

ldi		r16,3			;set up for 3 number entries
rcall	WriteEEPROM

inc		r30
ldi		r16,5
rcall	WriteEEPROM

inc		r30
ldi		r16,8
rcall	WriteEEPROM

inc		r30
ldi		r16,4
rcall	WriteEEPROM		;number of address now written

ldi		r16,6			;set up for six entries

ldi		r30,$0B
rcall	WriteEEPROM		;put in number of morse code entries

inc		r30
ldi		r16,26			;"C" -.-. (10 +16)
rcall	WriteEEPROM

inc		r30
ldi		r16,15			;"O" --- (7 + 8)
rcall	WriteEEPROM

inc		r30
ldi		r16,10			;"R" .-. (2 + 8)
rcall	WriteEEPROM

inc		r30
ldi		r16,3			;"T" - (1 + 2)
rcall	WriteEEPROM

inc		r30
ldi		r16, 2			;"E" . (0 + 2)
rcall	WriteEEPROM

inc		r30
ldi		r16,28			;"Z" --.. (12 + 16)
rcall	WriteEEPROM

ret


;--------------------------------
